/**
 * vim: set ts=4 :
 * =============================================================================
 * Auto Team Balance Immunity
 * Copyright (C) 2004-2008 AlliedModders LLC.  All rights reserved.
 * =============================================================================
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, version 3.0, as published by the
 * Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * As a special exception, AlliedModders LLC gives you permission to link the
 * code of this program (as well as its derivative works) to "Half-Life 2," the
 * "Source Engine," the "SourcePawn JIT," and any Game MODs that run on software
 * by the Valve Corporation.  You must obey the GNU General Public License in
 * all respects for all other code used.  Additionally, AlliedModders LLC grants
 * this exception to all derivative works.  AlliedModders LLC defines further
 * exceptions, found in LICENSE.txt (as of this writing, version JULY-31-2007),
 * or <http://www.sourcemod.net/license.php>.
 *
 * Version: $Id$
 */

#include "extension.h"

AutoTeamBalanceImmunity g_AutoTeamBalanceImmunity;
SMEXT_LINK(&g_AutoTeamBalanceImmunity);

IGameConfig *g_pGameConf = NULL;
IServerGameEnts *gameents = NULL;

SH_DECL_HOOK2_void(IServerGameClients, ClientPutInServer, SH_NOATTRIB, 0, edict_t *, char const *);
SH_DECL_HOOK1_void(IServerGameClients, ClientDisconnect, SH_NOATTRIB, 0, edict_t *);

SH_DECL_MANUALHOOK0(MHook_CanBeAutobalanced, 0, 0, 0, bool);

class BaseAccessor : public IConCommandBaseAccessor
{
public:
	bool RegisterConCommandBase(ConCommandBase *pCommandBase)
	{
		return META_REGCVAR(pCommandBase);
	}
} s_BaseAccessor;

ConVar g_version("tf_autoteambalanceimmunity_version", SMEXT_CONF_VERSION, FCVAR_PLUGIN|FCVAR_SPONLY|FCVAR_REPLICATED, "Auto Team Balance Immunity version");

bool AutoTeamBalanceImmunity::SDK_OnLoad(char *error, size_t maxlength, bool late)
{
	char conf_error[255] = "";

	if (!gameconfs->LoadGameConfigFile("tf2_autoteambalanceimmunity.offsets", &g_pGameConf, conf_error, sizeof(conf_error)))
	{
		if (conf_error[0])
		{
			g_pSM->Format(error, maxlength, "Could not read tf2_autoteambalanceimmunity.offsets.txt: %s", conf_error);
		}

		return false;
	}
	
	int offset;
	if (!g_pGameConf->GetOffset("CanBeAutobalanced", &offset))
	{
		g_pSM->Format(error, maxlength, "Error getting \"CanBeAutobalanced\" offset");
		return false;
	}
	SH_MANUALHOOK_RECONFIGURE(MHook_CanBeAutobalanced, offset, 0, 0);

	return true;
}

void AutoTeamBalanceImmunity::SDK_OnUnload()
{
	return;
}


void AutoTeamBalanceImmunity::OnClientPutInServer(edict_t *pEdict, char const *playername)
{
	IServerUnknown *unk = pEdict->GetUnknown();

	if (unk)
	{
		CBaseEntity *pBase = unk->GetBaseEntity();

		if (pBase)
		{
			SH_ADD_MANUALHOOK_MEMFUNC(MHook_CanBeAutobalanced, pBase, this, &AutoTeamBalanceImmunity::CanBeAutobalanced, false);
		}
	}
}

void AutoTeamBalanceImmunity::OnClientDisconnect(edict_t *pEdict)
{
	IServerUnknown *unk = pEdict->GetUnknown();

	if (unk)
	{
		CBaseEntity *pBase = unk->GetBaseEntity();

		if (pBase)
		{
			SH_REMOVE_MANUALHOOK_MEMFUNC(MHook_CanBeAutobalanced, pBase, this, &AutoTeamBalanceImmunity::CanBeAutobalanced, false);
		}
	}
}

bool AutoTeamBalanceImmunity::CanBeAutobalanced()
{
	CBaseEntity *pPlayer = META_IFACEPTR(CBaseEntity);
	edict_t *pEdict = gameents->BaseEntityToEdict(pPlayer);

	if (!pEdict)
		RETURN_META_VALUE(MRES_IGNORED, false);
	
	IGamePlayer *p = playerhelpers->GetGamePlayer(pEdict);
	
	if (!p)
		RETURN_META_VALUE(MRES_IGNORED, false);
	
	AdminId id = p->GetAdminId();
	
	if (id == INVALID_ADMIN_ID)
		RETURN_META_VALUE(MRES_IGNORED, false);
	
	if (adminsys->GetAdminFlags(id, Access_Effective) & DONATOR_FLAG)
	{
		RETURN_META_VALUE(MRES_SUPERCEDE, false);
	}

	RETURN_META_VALUE(MRES_IGNORED, 0);
}

bool AutoTeamBalanceImmunity::SDK_OnMetamodLoad(SourceMM::ISmmAPI *ismm, char *error, size_t maxlen, bool late)
{
	GET_V_IFACE_ANY(GetServerFactory, gameents, IServerGameEnts, INTERFACEVERSION_SERVERGAMEENTS);

	SH_ADD_HOOK_MEMFUNC(IServerGameClients, ClientPutInServer, serverclients, this, &AutoTeamBalanceImmunity::OnClientPutInServer, true);
	SH_ADD_HOOK_MEMFUNC(IServerGameClients, ClientDisconnect, serverclients, this, &AutoTeamBalanceImmunity::OnClientDisconnect, false);
	
	ConVar_Register(0, &s_BaseAccessor);

	return true;
}

bool AutoTeamBalanceImmunity::SDK_OnMetamodUnload(char *error, size_t maxlength)
{
	SH_REMOVE_HOOK_MEMFUNC(IServerGameClients, ClientPutInServer, serverclients, this, &AutoTeamBalanceImmunity::OnClientPutInServer, true);
	SH_REMOVE_HOOK_MEMFUNC(IServerGameClients, ClientDisconnect, serverclients, this, &AutoTeamBalanceImmunity::OnClientDisconnect, false);

	return true;
}